Type: Package
Package: pedFamilias
Title: Import and Export 'Familias' Files
Version: 0.2.5
Authors@R: 
    person("Magnus Dehli", "Vigeland", , "m.d.vigeland@medisin.uio.no", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-9134-4962"))
Description: Tools for exchanging pedigree data between the 'pedsuite'
    packages and the 'Familias' software for forensic kinship computations
    (Egeland et al. (2000) <doi:10.1016/s0379-0738(00)00147-x>). These
    functions were split out from the 'forrel' package to streamline
    maintenance and provide a lightweight alternative for packages otherwise
    independent of 'forrel'.
License: GPL (>= 3)
URL: https://github.com/magnusdv/pedFamilias
BugReports: https://github.com/magnusdv/pedFamilias/issues
Depends: pedtools, R (>= 4.2)
Imports: pedmut
Suggests: testthat
Encoding: UTF-8
Language: en-GB
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-17 12:03:17 UTC; magnu
Author: Magnus Dehli Vigeland [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9134-4962>)
Maintainer: Magnus Dehli Vigeland <m.d.vigeland@medisin.uio.no>
Repository: CRAN
Date/Publication: 2026-05-17 15:10:15 UTC
